/******************************************************************************
 *
 * FILE: emios.c
 *
 * Description:
 *    PWM generation with use of the eMIOS module
 *
 * History:
 *    06.03.2017   MF   Initial version
 *
 ******************************************************************************
 * Copyright (C) 2015-2018 HighTec EDV-Systeme GmbH.  All rights reserved.
 *
 * This is proprietary software. Any use of the software requires a written
 * license agreement with HighTec EDV-Systeme GmbH. Please contact for
 * detailed license information: info@hightec-rt.com.
 *****************************************************************************/

 /* BSP support package */
#include "bsp.h"

/* GTM timer header */
#include "emios.h"

uint32_t g_emios_ch0_pwmDuty = 1;
uint8_t g_emios_ch0_direction = 1;

/* ============================================================================
 *            ISR FUNCTIONS
 * ==========================================================================*/

/* ----------------------------------------------------------------------------
 * FUNCTION: EmiosCh1IsrHandler()
 *    Channel 1 ISR Handler
 * --------------------------------------------------------------------------*/

void EmiosCh1IsrHandler(void)
{
    uint32_t pwmDuty = g_emios_ch0_pwmDuty;
    uint8_t direction = g_emios_ch0_direction;

    /* Global variable check */
    if((pwmDuty < 1)||(pwmDuty > 99))
    {
    	pwmDuty = (uint32_t) 50; /* If g_emios_ch0_pwmDuty is out of the
    	                            range, default is 50% duty cycle*/
    }
    if(direction > 1)
    {
    	direction = (uint8_t) 0; /* If g_emios_ch0_direction is out of the
    	                            range, default is 0 increase duty */
    }

	/* Sweep PWM duty from 1% - 99% */
	/* increase duty */
	if (direction == 1) {
		if (pwmDuty < 99){
			pwmDuty++;
		}
		else{
			direction = 0;
		}
	}
	/* decrease duty */
	else{
		if (pwmDuty > 1){
			pwmDuty--;
		}
		else{
			direction = 1;
		}
	}

	/* Update PWM duty */
	EMIOS_0.CH[0].A.R = ((EMIOS_CH0_PERIODE_TICKS * pwmDuty)/100);
	EMIOS_0.CH[0].B.R = EMIOS_CH0_PERIODE_TICKS;

	g_emios_ch0_pwmDuty = pwmDuty;
	g_emios_ch0_direction = direction;

	EMIOS_0.CH[1].S.B.FLAG = 1; 		/* Clear interrupt flag */
}

/* ============================================================================
 * eMIOS example API
 * ==========================================================================*/

/* ----------------------------------------------------------------------------
 * Name:        emios_Init
 * Parameters:
 *              none
 * Returns:
 *              none
 * Description: Initialization of Global clock
 *				Initialization of Channel 0 and 1
 * --------------------------------------------------------------------------*/

void emios_Init(void)
{ 
	/* Global initialization eMIOS */
	EMIOS_0.MCR.B.GPREN = 0x0; /* Prescaler disabled (no clock) and prescaler
	                             counter is cleared */
	EMIOS_0.MCR.B.GPRE = EMIOS_EMIOSMCR_GPRE; /* Global Prescaler */

    /*Configuration Channel 0 */
	EMIOS_0.CH[0].C.B.UCPREN = 0; /* Prescaler disabled (no clock) */
	EMIOS_0.CH[0].C.B.MODE = 0x1U; /* GPIO mode, the registers A1 and B1 can
	                                  be set in this mode */
	EMIOS_0.CH[0].C.B.UCPRE = EMIOS_CH0_CTRL_CLK_SRC_SR;
	EMIOS_0.CH[0].C.B.BSL = 0x3; /* Bus Select: UC internal counter*/

	EMIOS_0.CH[0].A.R = EMIOS_CH0_PERIODE_TICKS/2;

	EMIOS_0.CH[0].B.R = EMIOS_CH0_PERIODE_TICKS;

	/*Configuration Channel 1 */
	EMIOS_0.CH[1].C.B.UCPREN = 0; /* Prescaler disabled (no clock) */
	EMIOS_0.CH[1].C.B.MODE = 0x1U; /* GPIO mode, the registers A1 and B1 can
	                                  be set in this mode */
	EMIOS_0.CH[1].C.B.UCPRE = EMIOS_CH1_CTRL_CLK_SRC_SR;
	EMIOS_0.CH[1].C.B.BSL = 0x3;    /* Bus Select: UC internal counter*/
	EMIOS_0.CH[1].C.B.FEN = 0x1U;   /* FLAG Enable bit, interrupt enable */

	EMIOS_0.CH[1].A.R = ((EMIOS_CH1_PERIODE_TICKS * EMIOS_CH1_DUTY)/100);

	EMIOS_0.CH[1].B.R = EMIOS_CH1_PERIODE_TICKS;

	/* Set PWM mode */
	EMIOS_0.CH[0].C.B.MODE = 0x58U; /* Output Pulse Width and Frequency
		                               Modulation Buffered (OPWFMB) mode */
	EMIOS_0.CH[1].C.B.MODE = 0x58U; /* Output Pulse Width and Frequency
		                               Modulation Buffered (OPWFMB) mode */

	/* Double buffer register A2 and B2 configuration for CH0 and CH1*/
	EMIOS_0.CH[0].A.R = EMIOS_CH0_PERIODE_TICKS/2;
    EMIOS_0.CH[0].B.R = EMIOS_CH0_PERIODE_TICKS;

	EMIOS_0.CH[1].A.R = ((EMIOS_CH1_PERIODE_TICKS * EMIOS_CH1_DUTY)/100);
	EMIOS_0.CH[1].B.R = EMIOS_CH1_PERIODE_TICKS;

	/* Register interrupt handler */
	bsp_isr_RegisterHandler(UC_EMIOS_CH1_ISR, 10, EmiosCh1IsrHandler);

	/* Configure eMIOS output pins */
	/* eMIOS_0 channel_0 port configuration PD[15] */
	/* enable output buffer, default (weak) drive, eMIOS function */
	bsp_pin_EnableOutput(EMIOS_CH0_PIN_PD15, EMIOS_CH_0_SSS);
	/* eMIOS_0 channel 1 port configuration PC[9] */
	/* enable output buffer, medium drive, eMIOS function */
	bsp_pin_EnableOutput(EMIOS_CH1_PIN_PC9, EMIOS_CH_1_SSS);

	/* Internal and global clock enable */
	EMIOS_0.CH[0].C.B.UCPREN = 1;
	EMIOS_0.CH[1].C.B.UCPREN = 1;
	EMIOS_0.MCR.B.GPREN = 1;
	EMIOS_0.MCR.B.GTBE = 1;
}
