/******************************************************************************
 *
 * FILE: emios.h
 *
 * Description:
 *    PWM generation with use of the eMIOS timer
 *
 * History:
 *    06.03.2018   MF   Initial version
 *
 ******************************************************************************
 * Copyright (C) 2015-2018 HighTec EDV-Systeme GmbH.  All rights reserved.
 *
 * This is proprietary software. Any use of the software requires a written
 * license agreement with HighTec EDV-Systeme GmbH. Please contact for
 * detailed license information: info@hightec-rt.com.
 *****************************************************************************/

#ifndef EMIOS_H
#define EMIOS_H

/* ============================================================================
 * eMIOS EXAMPLE SETTING
 * User ** can ** change it, here in this file
 * ==========================================================================*/

/* eMIOS Global clock */
#define EMIOS_GLOBAL_CLK                 5	    /* 10MHz */

/* eMIOS period
 * Note: Example changes duty-cycle in time
 * */
#define EMIOS_CH0_PERIODE_TIME_US        20      /* us */

/*EMIOS CH1 period and duty-cycle, generating the interrupt */
#define EMIOS_CH1_PERIODE_TIME_US        50000  /* us */
#define EMIOS_CH1_DUTY                   50     /* % */

/* ============================================================================
 * eMIOS, PIN & CLOCK INSTANCES IMPLEMENTATION
 * Only relevant configuration for BSP support package
 * ==========================================================================*/

#define EMIOS_CH_0_SSS	0x4	/* Port pin functionality */
#define EMIOS_CH_1_SSS	0xE	/* Port pin functionality */

/* eMIOS CH ports */
#define EMIOS_CH0_PIN_PD15		63
#define EMIOS_CH1_PIN_PC9	    41

/* eMIOS Module clock from AUX */
#define EMIOS_SYS_CLOCK           UC_EMIOS_CLK

/* EMIOS_GFR[F0,F1] ISR Vector (base vector for subsequent channels)  */
#define UC_EMIOS_CH1_ISR		    900

/* ============================================================================
 * eMIOS EXTERN
 * ==========================================================================*/

extern uint32_t g_emios_ch0_pwmDuty;  /* Duty cycle, range 1 to 99%*/
extern uint8_t g_emios_ch0_direction; /* Increasing or decreasing the duty
                                           cycle ( 1 or 0  ) */

/* ============================================================================
 * eMIOS API INTERFACE
 * ==========================================================================*/

void emios_Init(void);

/* ============================================================================
 * GTM CONFIGURATION
 * Example GTM setting
 * ==========================================================================*/
/* Selects the clock divider value for the global prescaler */
#define EMIOS_EMIOSMCR_GPRE     ((EMIOS_SYS_CLOCK/EMIOS_GLOBAL_CLK)-1)

/* CH0 UC internal prescaler clock divider */
#if((EMIOS_CH0_PERIODE_TIME_US * EMIOS_GLOBAL_CLK) < 0x10000U)
#define EMIOS_CH0_CTRL_CLK_SRC_SR    0 /* EMIOS_GLOBAL_CLK/1 */
#elif(((EMIOS_CH0_PERIODE_TIME_US * EMIOS_GLOBAL_CLK)/2) < 0x10000U)
#define EMIOS_CH0_CTRL_CLK_SRC_SR    1 /* EMIOS_GLOBAL_CLK/2 */
#elif(((EMIOS_CH0_PERIODE_TIME_US * EMIOS_GLOBAL_CLK)/3) < 0x10000U)
#define EMIOS_CH0_CTRL_CLK_SRC_SR    2 /* EMIOS_GLOBAL_CLK/3 */
#elif(((EMIOS_CH0_PERIODE_TIME_US * EMIOS_GLOBAL_CLK)/4) < 0x10000U)
#define EMIOS_CH0_CTRL_CLK_SRC_SR    3 /* EMIOS_GLOBAL_CLK/4 */
#else
#error eMIOS CH0 periode is out of range
#endif

/* CH1 UC internal prescaler clock divider */
#if((EMIOS_CH1_PERIODE_TIME_US * EMIOS_GLOBAL_CLK) < 0x10000U)
#define EMIOS_CH1_CTRL_CLK_SRC_SR    0 /* EMIOS_GLOBAL_CLK/1 */
#elif(((EMIOS_CH1_PERIODE_TIME_US * EMIOS_GLOBAL_CLK)/2) < 0x10000U)
#define EMIOS_CH1_CTRL_CLK_SRC_SR    1 /* EMIOS_GLOBAL_CLK/2 */
#elif(((EMIOS_CH1_PERIODE_TIME_US * EMIOS_GLOBAL_CLK)/3) < 0x10000U)
#define EMIOS_CH1_CTRL_CLK_SRC_SR    2 /* EMIOS_GLOBAL_CLK/3 */
#elif(((EMIOS_CH1_PERIODE_TIME_US * EMIOS_GLOBAL_CLK)/4) < 0x10000U)
#define EMIOS_CH1_CTRL_CLK_SRC_SR    3 /* EMIOS_GLOBAL_CLK/4 */
#else
#error eMIOS CH1 periode is out of range
#endif

#define EMIOS_CH0_PERIODE_TICKS \
	(uint16_t)((EMIOS_CH0_PERIODE_TIME_US * EMIOS_GLOBAL_CLK)/ \
			   (EMIOS_CH0_CTRL_CLK_SRC_SR + 1))
#define EMIOS_CH1_PERIODE_TICKS \
	(uint16_t)((EMIOS_CH1_PERIODE_TIME_US * EMIOS_GLOBAL_CLK)/ \
			   (EMIOS_CH1_CTRL_CLK_SRC_SR + 1))

/* ============================================================================
 * eMIOS CONFIGURATION CHECKS
 * ==========================================================================*/

/* Duty cycle for CH1 - <1% to 99%> */
#if ((EMIOS_CH1_DUTY < 1) || (EMIOS_CH1_DUTY > 99))
#error Duty cycle is out of the range 1 to 99%
#endif


/* GPRE whole integer divider check */
#if ((EMIOS_SYS_CLOCK) - \
	 (EMIOS_GLOBAL_CLK * (EMIOS_EMIOSMCR_GPRE + 1)))
#error FXCLK clock is not exact, divider is not a whole number. Check the setting
#endif

#endif /* EMIOS_H */
